#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dotenv = require("dotenv");
var fs = require("fs");
var Path = require("path");
var ConsolePrinter_1 = require("./ConsolePrinter");
var VirtualDeviceScript_1 = require("./VirtualDeviceScript");
dotenv.config();
var TestRunner = {
    addTokens: function (script) {
        for (var _i = 0, _a = Object.keys(process.env); _i < _a.length; _i++) {
            var key = _a[_i];
            if (key.startsWith("token.") || key.startsWith("replace.")) {
                var value = process.env[key];
                var token = key.substr(key.indexOf(".") + 1);
                console.log("Replacing token: " + token + " with: " + value);
                script.findReplace(token, value);
            }
        }
    },
    run: function (path) {
        var script = new VirtualDeviceScript_1.VirtualDeviceScript(process.env.VIRTUAL_DEVICE_TOKEN, process.env.BESPOKEN_USER_ID, (process.env.BATCH_MODE !== undefined));
        TestRunner.addTokens(script);
        script.on("result", function (error, resultItem) {
            console.log("ResultItem: " + JSON.stringify(resultItem, null, 2));
        });
        var printer = new ConsolePrinter_1.ConsolePrinter();
        var directory = ".";
        var fileTest;
        if (path) {
            if (!fs.existsSync(path)) {
                throw new Error(Path.resolve(path) + " does not exist");
            }
            if (fs.statSync(path).isDirectory()) {
                directory = path;
            }
            else {
                fileTest = path;
            }
        }
        if (fileTest) {
            console.log("Running Test:" + fileTest);
            script.executeFile(fileTest).then(function (result) {
                console.log(printer.printResult(fileTest, result));
                if (result.result !== "success") {
                    process.exit(1);
                }
            });
        }
        else {
            console.log("Running Tests from: " + Path.resolve(directory));
            script.executeDir(directory).then(function (results) {
                console.log(printer.printResultsByFile(results));
                for (var _i = 0, _a = Object.keys(results); _i < _a.length; _i++) {
                    var resultFile = _a[_i];
                    var result = results[resultFile];
                    if (result.result !== "success") {
                        process.exit(1);
                    }
                }
            });
        }
    },
};
process.on("unhandledRejection", function (error) {
    console.log("unhandledRejection", error);
});
if (process.argv.length < 3) {
    console.log("");
    console.log("Bespoken Virtual Device test runner installed!");
    console.log("");
    process.exit(0);
}
var file = process.argv.length > 2 ? process.argv[2] : undefined;
TestRunner.run(file);
//# sourceMappingURL=ScriptRunner.js.map