"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.retry = void 0;
var retrier = require("retry");
exports.retry = function (fn, opts) {
    var run = function (resolve, reject) {
        var options = opts || {};
        var op;
        if (!("randomize" in options)) {
            options.randomize = true;
        }
        op = retrier.operation(options);
        function bail(err) {
            reject(err || new Error("Aborted"));
        }
        function onError(err, num) {
            if (err.bail) {
                bail(err);
                return;
            }
            if (!op.retry(err)) {
                reject(op.mainError());
            }
        }
        function runAttempt(num) {
            var val;
            try {
                val = fn(bail, num);
            }
            catch (err) {
                onError(err, num);
                return;
            }
            Promise.resolve(val)
                .then(resolve)
                .catch(function catchIt(err) {
                onError(err, num);
            });
        }
        op.attempt(runAttempt);
    };
    return new Promise(run);
};
//# sourceMappingURL=util.js.map